@@ -89,7 +89,7 @@ RS_XML(Parse)(USER_OBJECT_ fileName, USER_OBJECT_ handlers, USER_OBJECT_ endElem
                  USER_OBJECT_ trim, USER_OBJECT_ useExpat, USER_OBJECT_ stateObject,
                   USER_OBJECT_ replaceEntities, USER_OBJECT_ validate, USER_OBJECT_ saxVersion,
 	      USER_OBJECT_ branches, USER_OBJECT_ useDotNames, USER_OBJECT_ errorFun,
-              USER_OBJECT_ manageMemory)
+              USER_OBJECT_ manageMemory, USER_OBJECT_ r_encoding)
 {
 #ifdef LIBEXPAT
   FILE *file = NULL;
@@ -168,7 +168,7 @@ RS_XML(Parse)(USER_OBJECT_ fileName, USER_OBJECT_ handlers, USER_OBJECT_ endElem
    xmlSubstituteEntitiesDefault(LOGICAL_DATA(replaceEntities)[0]);   
 #endif
 
-  status = RS_XML(libXMLEventParse)(input, parserData, asTextBuffer, INTEGER_DATA(saxVersion)[0]);
+  status = RS_XML(libXMLEventParse)(input, parserData, asTextBuffer, INTEGER_DATA(saxVersion)[0], r_encoding);
 
 /* How about using R_alloc() here so that it is freed, i.e. for the fileName and the parserData itself. */
   ans = parserData->stateObject ? parserData->stateObject : handlers;
@@ -178,9 +178,9 @@ RS_XML(Parse)(USER_OBJECT_ fileName, USER_OBJECT_ handlers, USER_OBJECT_ endElem
   if(parserData->stateObject && parserData->stateObject != NULL_USER_OBJECT)
      R_ReleaseObject(parserData->stateObject);
 
-  if(status != 0) {
-    RSXML_structuredStop(errorFun, NULL);
-  }
+  if(status != 0) 
+     RSXML_structuredStop(errorFun, NULL);
+
 
   /* free(parserData); Now using R_alloc */
 
@@ -217,10 +217,10 @@ RS_XML(entityDeclarationHandler)(void *userData, const XML_Char *entityName,
    SET_VECTOR_ELT(opArgs, i,  NEW_CHARACTER(1));
    SET_STRING_ELT(VECTOR_ELT(opArgs, i), 0, ENC_COPY_TO_USER_STRING(xml_args[i] ? (const xmlChar *)xml_args[i] :  (const xmlChar *)"")); 
   }
-  UNPROTECT(1);
 
   RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "entityDeclaration"), 
                            (const char*)NULL, parserData, opArgs);
+  UNPROTECT(1);
 }
 
 
@@ -298,8 +298,9 @@ void RS_XML(endElement)(void *userData, const char *name)
 
   fun = findEndElementFun(name, rinfo);
   if(fun)  {
-      USER_OBJECT_ val = RS_XML(invokeFunction)(fun, opArgs, rinfo->stateObject, rinfo->ctx);
+      USER_OBJECT_ val = PROTECT(RS_XML(invokeFunction)(fun, opArgs, rinfo->stateObject, rinfo->ctx));
       updateState(val, rinfo);
+      UNPROTECT(1);
   }
   else
      RS_XML(callUserFunction)(HANDLER_FUN_NAME(rinfo, "endElement"), NULL, ((RS_XMLParserData*) userData), opArgs);
@@ -358,7 +359,7 @@ fixedTrim(char *str,  int len, int *start, int *end)
     return(str);
 
    /* Jump to the end */
-  tmp = str + len - 2;
+  tmp = str + len - 2;  // DTL has 1
   while(tmp >= str && isspace(*tmp)) {
       tmp--;
       (*end)--;
@@ -385,6 +386,7 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
  USER_OBJECT_ opArgs = NULL;
  RS_XMLParserData *parserData = (RS_XMLParserData*)userData; 
  DECL_ENCODING_FROM_EVENT_PARSER(parserData)
+ int nprot = 0;
 
    /* XXX Here is where we have to ignoreBlankLines and use the trim setting in parserData */
   if(parserData->current) {
@@ -428,13 +430,13 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
  
   if(parserData->trim) {
     tmpString = trim(tmpString);
-    len = strlen(tmpString);
+    len = (int) strlen(tmpString);
   }
 
   if(len > 0 || parserData->ignoreBlankLines == 0 ) {
-    PROTECT(opArgs = NEW_LIST(1));
-     SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(tmpString));
+    PROTECT(opArgs = NEW_LIST(1)); nprot++;
+    SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
+    SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(tmpString));
   }
 
   free(tmp);
@@ -444,9 +446,9 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
      */
 
   if(opArgs != NULL) {
-      RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "text"), (const char *)NULL, ((RS_XMLParserData*) userData), opArgs);
-     UNPROTECT(1);
+    RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "text"), (const char *)NULL, ((RS_XMLParserData*) userData), opArgs);
   }
+  UNPROTECT(nprot);
 }
 
 
@@ -509,8 +511,9 @@ RS_XML(callUserFunction)(const char *opName, const char *preferredName, RS_XMLPa
    return(NULL_USER_OBJECT);
   }
 
-  val = RS_XML(invokeFunction)(fun, opArgs, parserData->stateObject, parserData->ctx);
+  val = PROTECT(RS_XML(invokeFunction)(fun, opArgs, parserData->stateObject, parserData->ctx));
   updateState(val, parserData);
+  UNPROTECT(1);
   return(val); 
 }
 
