@@ -161,7 +161,7 @@ static R_CallMethodDef callMethods[] = {
 #endif
 	ENTRY(RS_XML_getDTD, 5),
 	ENTRY(RS_XML_libxmlVersion, 0),
-	ENTRY(RS_XML_Parse, 17),
+	ENTRY(RS_XML_Parse, 18),
 	ENTRY(RS_XML_ParseTree, 21),
 	ENTRY(R_newXMLDtd, 5),
 	ENTRY(R_newXMLDoc, 3),
@@ -201,7 +201,7 @@ static R_CallMethodDef callMethods[] = {
 	ENTRY(RS_XML_setKeepBlanksDefault, 1),
 	ENTRY(R_getDocEncoding, 1),
 	ENTRY(R_getLineNumber, 1),
-        ENTRY(RS_XML_xpathEval, 7),
+        ENTRY(RS_XML_xpathEval, 9),
         ENTRY(RS_XML_xmlNodeChildrenReferences, 3),
 	ENTRY(RS_XML_freeDoc, 1),
 	ENTRY(RS_XML_setRootNode, 2),
@@ -228,7 +228,7 @@ static R_CallMethodDef callMethods[] = {
 	ENTRY(RS_XML_xmlNodeName, 1),
 	ENTRY(RS_XML_xmlNodeNamespace, 1),
 	ENTRY(RS_XML_xmlNodeAttributes, 3),
-	ENTRY(RS_XML_xmlNodeChildrenReferences, 3),
+	// ENTRY(RS_XML_xmlNodeChildrenReferences, 3), // duplicate
 	ENTRY(R_xmlNodeValue, 3),
 	ENTRY(R_setXMLInternalTextNode_value, 2),
 	ENTRY(RS_XML_xmlNodeParent, 2),
@@ -245,6 +245,7 @@ static R_CallMethodDef callMethods[] = {
 	ENTRY(RS_XML_xmlXIncludeProcessFlags, 2),
 	ENTRY(RS_XML_xmlXIncludeProcessTreeFlags, 2),
 	ENTRY(R_convertXMLNsRef, 1),
+	ENTRY(R_replaceNodeWithChildren, 1),
 	{NULL, NULL, 0}
 };
 
@@ -290,7 +291,7 @@ RS_XML_getStructuredErrorHandler()
     SEXP ans;
     PROTECT(ans = NEW_LIST(2));
     SET_VECTOR_ELT(ans, 0, R_MakeExternalPtr(xmlGenericErrorContext, Rf_install("xmlGenericErrorContext"), R_NilValue));
-    SET_VECTOR_ELT(ans, 1, R_MakeExternalPtr(xmlStructuredError, Rf_install("xmlStructuredErrorFunc"), R_NilValue));
+    SET_VECTOR_ELT(ans, 1, R_MakeExternalPtr((void *)xmlStructuredError, Rf_install("xmlStructuredErrorFunc"), R_NilValue));
     UNPROTECT(1);
     return(ans);
 }
@@ -338,19 +339,19 @@ CreateCharSexpWithEncoding(const xmlChar *encoding, const xmlChar *str)
 #ifdef HAVE_R_CETYPE_T
     cetype_t enc = CE_NATIVE;
 
-    if(encoding == (const xmlChar *) NULL || encoding == (const xmlChar *) "") {
+    if(encoding == (const xmlChar *) NULL || xmlStrcmp(encoding, (const xmlChar *) "")) {
   	    enc = CE_NATIVE;
-    } else if(xmlStrcmp(encoding, "UTF-8") == 0 || xmlStrcmp(encoding, "utf-8") == 0)
+    } else if(xmlStrcmp(encoding, (xmlChar *)"UTF-8") == 0 || xmlStrcmp(encoding, (xmlChar *)"utf-8") == 0)
 	    enc = CE_UTF8;
-    else if(xmlStrcmp(encoding, "ISO-8859-1") == 0 || xmlStrcmp(encoding, "iso-8859-1") == 0)
+    else if(xmlStrcmp(encoding, (xmlChar *)"ISO-8859-1") == 0 || xmlStrcmp(encoding, (xmlChar *)"iso-8859-1") == 0)
 	    enc = CE_LATIN1;
     else {
-	str = translateChar(mkChar(str));
+	str = (xmlChar *)translateChar(mkChar((const char *) str));
     }
 // REprintf("encoding: %d\n", enc);
-    ans = mkCharCE(str, enc);
+    ans = mkCharCE((const char *) str, enc);
 #else
-    ans = mkChar(str);
+    ans = mkChar((const char *) str);
 #endif
     return(ans);
 }
@@ -361,7 +362,7 @@ R_lookString(SEXP rstr)
 {
     const char *str;
     str = CHAR(STRING_ELT(rstr, 0));
-    return(ScalarInteger(strlen(str)));
+    return(ScalarInteger((int) strlen(str)));
 }
 
 
