@@ -80,7 +80,7 @@ createSAX2AttributesList(const xmlChar **attributes, int nb_attributes, int nb_d
       char *tmp;
       int len;
 
-      len = (ptr[4] - ptr[3] + 1);
+      len = (int)(ptr[4] - ptr[3] + 1);
       tmp = malloc(sizeof(char) * len);
       if(!tmp) {
          PROBLEM "Cannot allocate space for attribute of length %d", (int) (ptr[4] - ptr[3] + 2)
@@ -132,11 +132,12 @@ typedef struct {
 int
 RS_XML_readConnectionInput(void *context, char *buffer, int len)
 {
-  SEXP e, tmp, arg;
+  SEXP e, tmp, arg = R_NilValue /* -Wall */;
   int n;
   int errorOccurred;
   const char *str;
   int left = len-1, count;
+  int nprot = 0;
 
 #ifdef R_XML_DEBUG
   char *orig = buffer;
@@ -162,12 +163,11 @@ RS_XML_readConnectionInput(void *context, char *buffer, int len)
      if they gave us a connection. */
   if(isFunction(fun)) {
      /* Invoke the user-provided function to get the next line. */
-    PROTECT(e = allocVector(LANGSXP, 2));
+    PROTECT(e = allocVector(LANGSXP, 2)); nprot++;
     SETCAR(e, fun);
-    PROTECT(arg = NEW_INTEGER(1));
+    PROTECT(arg = NEW_INTEGER(1)); nprot++;
     INTEGER_DATA(arg)[0] = len;
     SETCAR(CDR(e), arg);
-    UNPROTECT(1);
   } else 
       e = fun;
 
@@ -183,7 +183,7 @@ RS_XML_readConnectionInput(void *context, char *buffer, int len)
    tmp = R_tryEval(e, R_GlobalEnv, &errorOccurred);
 
    if(errorOccurred || !IS_CHARACTER(tmp)) {
-     UNPROTECT(1);
+     UNPROTECT(nprot);
      if ((ctx->sax != NULL) && (ctx->sax->error != NULL))  /* throw an XML error. */
            ctx->sax->error(ctx->userData, "Failed to call read on XML connection");
      return(-1);
@@ -192,7 +192,7 @@ RS_XML_readConnectionInput(void *context, char *buffer, int len)
    if(GET_LENGTH(tmp)) {
 
       str = CHAR_DEREF(STRING_ELT(tmp, 0));
-      n = strlen(str);
+      n = (int)strlen(str);
 
 
       if(n != 0) { /* Just add a new line and do it again. */
@@ -222,7 +222,7 @@ RS_XML_readConnectionInput(void *context, char *buffer, int len)
 /*  fprintf(stderr, "size (n=%d, count=%d) %s '%s'\n", n, count, str, orig);fflush(stderr); */
 #endif
 
-  UNPROTECT(1);
+  UNPROTECT(nprot);
 
 
   return(count);
@@ -265,7 +265,7 @@ RS_XML_xmlCreateConnectionParserCtxt(USER_OBJECT_ con)
 
 int
 RS_XML(libXMLEventParse)(const char *fileName, RS_XMLParserData *parserData, RS_XML_ContentSourceType asText,
-                          int saxVersion)
+			 int saxVersion, USER_OBJECT_ r_encoding)
 {
  xmlSAXHandlerPtr xmlParserHandler;
  xmlParserCtxtPtr ctx; 
@@ -305,6 +305,11 @@ RS_XML(libXMLEventParse)(const char *fileName, RS_XMLParserData *parserData, RS_
   ctx->userData = parserData;
   ctx->sax = xmlParserHandler;
 
+  if(Rf_length(r_encoding) && STRING_ELT(r_encoding, 0) != R_NaString) { 
+//      Rf_PrintValue(r_encoding);
+      ctx->encoding = xmlStrdup((const xmlChar *)CHAR(STRING_ELT(r_encoding, 0)));
+  }
+
   status = xmlParseDocument(ctx);
 
   ctx->sax = NULL;
@@ -343,7 +348,7 @@ getPropertyValue(const xmlChar **ptr)
   int len;
   char *tmp;
 
-      len = (ptr[4] - ptr[3] + 1);
+  len = (int)(ptr[4] - ptr[3] + 1);
       tmp = malloc(sizeof(char) * len);
       if(!tmp) {
          PROBLEM "Cannot allocate space for attribute of length %d", (int) (ptr[4] - ptr[3] + 2)
@@ -380,7 +385,7 @@ R_processBranch(RS_XMLParserData * rinfo,
 	  const xmlChar **ptr = p;
 	  for(i = 0; i < nb_attributes; i++, ptr += 5) {
 	    /*XXX does this get freed later on?*/
-            xmlSetProp(node, xmlStrdup(ptr[0]), getPropertyValue(ptr));
+            xmlSetProp(node, xmlStrdup(ptr[0]), (const xmlChar *)getPropertyValue(ptr));
 	  }
 	}
     }
@@ -439,7 +444,7 @@ R_endBranch(RS_XMLParserData *rinfo,
 
 	    PROTECT(args = NEW_LIST(1));
 	    if(tmp->doc == NULL) {
-		doc = xmlNewDoc("1.0");
+		doc = xmlNewDoc((const xmlChar*) "1.0");
 		initDocRefCounter(doc);
    	        xmlDocSetRootElement(doc, tmp);
 /*		fprintf(stderr, "<r:createDoc addr='%p'/>\n", doc); */
@@ -525,7 +530,7 @@ RS_XML(xmlSAX2StartElementNs)(void * userData,
   PROTECT(tmp = NEW_CHARACTER(1));
   if(URI) {
      SET_STRING_ELT(tmp, 0, ENC_COPY_TO_USER_STRING(XMLCHAR_TO_CHAR(URI))); 
-     SET_NAMES(tmp, ScalarString(CreateCharSexpWithEncoding(encoding,  ( (void*)prefix ? XMLCHAR_TO_CHAR(prefix) : "")))); 
+     SET_NAMES(tmp, ScalarString(CreateCharSexpWithEncoding(encoding,  ( (void*)prefix ? prefix : (const xmlChar *)"")))); 
   }
   SET_VECTOR_ELT(opArgs, 2, tmp);
   UNPROTECT(1);
@@ -576,15 +581,16 @@ RS_XML(xmlSAX2EndElementNs)(void * ctx,
   PROTECT(args = NEW_LIST(2));
   SET_VECTOR_ELT(args, 0, ScalarString(ENC_COPY_TO_USER_STRING(localname)));
 
-  PROTECT(tmp = ScalarString(ENC_COPY_TO_USER_STRING((XMLCHAR_TO_CHAR(URI)) ? XMLCHAR_TO_CHAR(URI) : ""))); 
+  PROTECT(tmp = ScalarString(ENC_COPY_TO_USER_STRING(URI ? URI : (const xmlChar *)""))); 
   if(prefix)
       SET_NAMES(tmp, ScalarString(ENC_COPY_TO_USER_STRING(prefix)));
   SET_VECTOR_ELT(args, 1, tmp);
 
-  fun = findEndElementFun(localname, rinfo);
+  fun = findEndElementFun((const char *)localname, rinfo);
   if(fun)  {
-      USER_OBJECT_ val = RS_XML(invokeFunction)(fun, args, rinfo->stateObject, rinfo->ctx);
+      USER_OBJECT_ val = PROTECT(RS_XML(invokeFunction)(fun, args, rinfo->stateObject, rinfo->ctx));
       updateState(val, rinfo);
+      UNPROTECT(1);
   } else
       RS_XML(callUserFunction)(HANDLER_FUN_NAME(ctx, "endElement"), NULL, (RS_XMLParserData *)ctx, args);
 
@@ -593,10 +599,12 @@ RS_XML(xmlSAX2EndElementNs)(void * ctx,
 
 
 
+#if 0
 static void
 RS_XML(xmlSAX2StartDocument)(void *userData)
 {
 }
+#endif
 
 void
 RS_XML(initXMLParserHandler)(xmlSAXHandlerPtr xmlParserHandler, int saxVersion)
@@ -790,7 +798,7 @@ do_getEntityHandler(void *userData, const xmlChar *name, const char * r_funName)
 	    ans->name = xmlStrdup(name);
 	    ans->orig = NULL; // xmlStrdup(CHAR_TO_XMLCHAR(value));
 	    ans->content = xmlStrdup(CHAR_TO_XMLCHAR(value));	    
-	    ans->length = strlen(value);
+	    ans->length = (int)strlen(value);
 #ifndef NO_CHECKED_ENTITY_FIELD
 	    ans->checked = 1;
 #endif
